MoveBox = class(Turbine.UI.Window)

function MoveBox:Constructor(parent, width, height, name)
    Turbine.UI.Control.Constructor(self)

    self:SetParent(parent)
    self:SetSize(width, height)
    self:SetVisible(false)
    self:SetMouseVisible(true)

    local OpacityBox = Turbine.UI.Control()
    OpacityBox:SetParent(self)
    OpacityBox:SetSize(width - 4, height - 4)
    OpacityBox:SetPosition(2, 2)
    OpacityBox:SetBackColor(Turbine.UI.Color(0.2, 0, 0, 0))
    OpacityBox:SetVisible(true)
    OpacityBox:SetMouseVisible(false)
    OpacityBox:SetOpacity(0.5)

    local AnchorBox = Turbine.UI.Control()
    AnchorBox:SetParent(self)
    AnchorBox:SetSize(math.min(width, 200), math.min(height, 30))
    AnchorBox:SetPosition(width / 2 - math.min(width, 200) / 2, height / 2 - math.min(height, 30) / 2)
    AnchorBox:SetBackColor(Turbine.UI.Color(1, 1, 1))
    AnchorBox:SetMouseVisible(false)

    local BoxLabel = Turbine.UI.Label()
    BoxLabel:SetParent(AnchorBox)
    BoxLabel:SetSize(math.min(width, 200) - 2, math.min(height, 30) - 2)
    BoxLabel:SetPosition(1, 1)
    BoxLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    BoxLabel:SetBackColor(Turbine.UI.Color(0, 0, 0))
    BoxLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    BoxLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    BoxLabel:SetMouseVisible(false)
    BoxLabel:SetText(name)

    local moving = false
    local moveX, moveY = 0, 0
    local parentVisibleMemory = parent:IsVisible()

    self.MouseDown = function(sender, args)
        moving = true
        moveX = args.X
        moveY = args.Y
    end

    self.MouseUp = function()
        moving = false
    end

    self.MouseMove = function(sender, args)
        if moving then
            local newLeft = self:GetLeft() - (moveX - args.X)
            local newTop = self:GetTop() - (moveY - args.Y)
            self:SetPosition(newLeft, newTop)
            parent:SetPosition(newLeft, newTop)
        end
    end

    self.ToggleMove = function()
        if not self:IsVisible() then
            parentVisibleMemory = parent:IsVisible()
        end
        self:SetVisible(not self:IsVisible())
        parent:SetVisible(not self:IsVisible() and parentVisibleMemory)
    end

    self.MouseEnter = function()
        OpacityBox:SetBackColor(Turbine.UI.Color(0.2, 0.375, 0.375, 0.375))
        BoxLabel:SetBackColor(Turbine.UI.Color(0.25, 0.25, 0.25))
    end

    self.MouseLeave = function()
        OpacityBox:SetBackColor(Turbine.UI.Color(0.2, 0, 0, 0))
        BoxLabel:SetBackColor(Turbine.UI.Color(0, 0, 0))
    end
end


function MoveBox:Unload()
  self:SetVisible(false)
  self:SetParent(nil)
  local controlList = self:GetControls()
  for i = 1, controlList:GetCount() do
      controlList:RemoveAt(1)
  end
  for key, _ in pairs(self) do
      self[key] = nil
  end
  collectgarbage()
end